# -*- coding: us-ascii -*-
"""Contrail NRPE base class module."""

import os
import shutil

from charmhelpers.contrib.charmsupport import nrpe
from charmhelpers.core import hookenv


class ContrailNrpeBase:
    """Base class for installing Contrail NRPE plugins."""

    CROND_DIR = "/etc/cron.d"
    CROND_FILE = ""  # Override in subclass
    CRONJOB_SCRIPT = ""  # Override in subclass
    CRONJOB_SCRIPT_ARGS = ""  # Override in subclass
    CRONJOB_SCRIPT_MODE = 0o100755
    CRONJOB_SCRIPT_UID = 0
    CRONJOB_SCRIPT_GID = 0
    NRPE_PLUGINS_DIR = "/usr/local/lib/nagios/plugins"
    NRPE_PLUGINS_MODE = 0o100755
    NRPE_PLUGINS_UID = 0
    NRPE_PLUGINS_GID = 0
    SHORTNAME = ""  # Override in subclass
    SUDOERS_DIR = "/etc/sudoers.d"
    SUDOERS_FILE = ""  # Override in subclass
    SUDOERS_MODE = 0o100440
    SUDOERS_UID = 0
    SUDOERS_GID = 0

    nrpe_opts = ""

    def __init__(self, nrpe_opts=""):
        """Initialize the object."""
        self._nagios_hostname = nrpe.get_nagios_hostname()
        self._files_dir = os.path.join(hookenv.charm_dir(), "files")
        self._nrpe_script = "check_{}".format(self.SHORTNAME)
        self.nrpe_opts = nrpe_opts

    def _install_nrpe_plugin(self):
        src = os.path.join(self._files_dir, "plugins", self._nrpe_script)
        os.makedirs(self.NRPE_PLUGINS_DIR, exist_ok=True)
        dst = shutil.copy(src, self.NRPE_PLUGINS_DIR)
        os.chmod(dst, self.NRPE_PLUGINS_MODE)
        os.chown(dst, uid=self.NRPE_PLUGINS_UID, gid=self.NRPE_PLUGINS_GID)
        hookenv.log(
            "NRPE script for {} installed at as {}".format(self.SHORTNAME, dst),
            hookenv.DEBUG,
        )
        return dst

    def _remove_nrpe_plugin(self):
        plugin_path = os.path.join(self.NRPE_PLUGINS_DIR, self._nrpe_script)
        if not os.path.exists(plugin_path):
            return
        os.remove(plugin_path)
        hookenv.log("deleted NRPE script {}".format(self.SHORTNAME), hookenv.DEBUG)

    def _install_sudoer(self):
        # User nagios is normally not able to run ipmi-sel and ipmi-sensors, so
        # we need to add a sudoer entry for it
        src = os.path.join(self._files_dir, self.SUDOERS_FILE)
        os.makedirs(self.SUDOERS_DIR, exist_ok=True)
        dst = shutil.copy(src, self.SUDOERS_DIR)
        os.chmod(dst, self.SUDOERS_MODE)
        os.chown(dst, uid=self.SUDOERS_UID, gid=self.SUDOERS_GID)
        hookenv.log(
            "sudoer file for {} installed at {}".format(self.SHORTNAME, dst),
            hookenv.DEBUG,
        )
        return dst

    def _remove_sudoer(self):
        sudoer_path = os.path.join(self.SUDOERS_DIR, self.SUDOERS_FILE)
        if not sudoer_path.exists():
            return
        sudoer_path.unlink()
        hookenv.log("deleted sudoer file: {}".format(sudoer_path), hookenv.DEBUG)

    def configure_nrpe_check(self, nrpe_setup):
        """Set up the NRPE check for the plugin."""
        cmd = " ".join([self._nrpe_script, self.nrpe_opts])
        nrpe_setup.add_check(
            shortname=self.SHORTNAME,
            description="{} Contrail NRPE check",
            check_cmd=cmd,
        )
        nrpe_setup.write()
        hookenv.log(
            "configured NRPE check for [{}]".format(self.SHORTNAME), hookenv.DEBUG
        )

    def remove_nrpe_check(self):
        """Remove the NRPE check for the plugin."""
        nrpe_setup = nrpe.NRPE(hostname=self._nagios_hostname, primary=False)
        cmd = " ".join([self._nrpe_script, self.nrpe_opts])
        nrpe_setup.remove_check(shortname=self.SHORTNAME, check_cmd=cmd)
        nrpe_setup.write()
        hookenv.log("removed NRPE check for {}".format(self.SHORTNAME), hookenv.DEBUG)

    def install(self):
        """Install the plugin."""
        self._install_nrpe_plugin()
        hookenv.log("Installed {}".format(self.SHORTNAME))

    def remove(self):
        """Remove the plugin and check."""
        self.remove_nrpe_check()
        self._remove_nrpe_plugin()
        hookenv.log("Removed {}".format(self.SHORTNAME))
