"""Contrail RabbitMQ check class module."""

import binascii
import os
import shlex
import shutil
import subprocess
from pathlib import Path

from charmhelpers.core import hookenv

from contrailnrpe.contrailbase import ContrailNrpeBase


class ContrailRabbitMqCheck(ContrailNrpeBase):
    """Class for installing the Contrail RabbitMQ NRPE plugin."""

    SHORTNAME = "rabbitmq_queues"
    CRONJOB_SCRIPT = "collect_rabbitmq_stats.sh"
    CROND_FILE = "rabbitmq-stats"
    SUDOERS_FILE = None  # unused

    def _install_cronjob(self, cron_user="root"):
        # Copy the cronjob script to the nagios plugins directory
        src = Path(self._files_dir) / self.CRONJOB_SCRIPT
        dst = Path(self.NRPE_PLUGINS_DIR) / self.CRONJOB_SCRIPT
        shutil.copy(str(src), str(dst))
        dst.chmod(self.CRONJOB_SCRIPT_MODE)
        os.chown(str(dst), uid=self.CRONJOB_SCRIPT_UID, gid=self.CRONJOB_SCRIPT_GID)
        hookenv.log(
            "Cronjob script [{}] copied to {}"
            "".format(self.CRONJOB_SCRIPT, self.NRPE_PLUGINS_DIR),
            hookenv.DEBUG,
        )

        cmdline = [str(dst)]
        if self.CRONJOB_SCRIPT_ARGS and isinstance(self.CRONJOB_SCRIPT_ARGS, str):
            cmdline.extend(
                [shlex.quote(arg) for arg in self.CRONJOB_SCRIPT_ARGS.split()]
            )
        else:
            # Run it once to generate the temp file, otherwise the nrpe check
            # might fail at first. For security reasons, cronjobs that allow
            # parameters shared by the user don't run the script as root
            # (check_ipmi allows a missing output file)
            subprocess.call(cmdline)

        # Generate random cronjob execution (internal in minutes)
        cron_interval = 5
        minutes_offsets = []
        minute_num = binascii.crc_hqx("".join(cmdline).encode(), 0) % cron_interval
        while minute_num < 60:
            minutes_offsets.append(str(minute_num))
            minute_num += cron_interval
        cronjob_line = "{minutes} * * * * {user} {cmd}\n".format(
            minutes=",".join(minutes_offsets), user=cron_user, cmd=" ".join(cmdline)
        )

        crond_file = Path(self.CROND_DIR) / self.CROND_FILE
        with crond_file.open("w") as crond_fd:
            crond_fd.write(cronjob_line)
            hookenv.log("Cronjob configured at {}".format(crond_file), hookenv.DEBUG)
        return dst

    def _remove_cronjob(self):
        crond_file = Path(self.CROND_DIR) / self.CROND_FILE
        cronjob_script = Path(self.NRPE_PLUGINS_DIR) / self.CRONJOB_SCRIPT
        for filename in (crond_file, cronjob_script):
            if not filename.exists():
                continue
            filename.unlink()
        hookenv.log(
            "Removed cronjob files [{}, {}]" "".format(crond_file, cronjob_script),
            hookenv.DEBUG,
        )

    def install(self):
        """Install the plugin and its cron job."""
        super(ContrailRabbitMqCheck, self).install()
        self._install_cronjob()

    def remove(self):
        """Remove the plugin, check and cron job."""
        super(ContrailRabbitMqCheck, self).remove()
        self._remove_cronjob()
