"""Contrail NRPE layer."""
import os
import subprocess

from charmhelpers.contrib.charmsupport import nrpe
from charmhelpers.core import hookenv

from charms.layer import status
from charms.reactive import hook, when, when_not
from charms.reactive.flags import clear_flag, is_flag_set, set_flag

from contrailnrpe import contrailrabbit, redis

import yaml


def detect_docker_containers():
    """Collect a list of the containers on the machine.

    In order to determine which containers we need to monitor we need to list
    the containers and translate that to a list of things to check.

    Returns a list of applications to monitor.
    """
    # docker list, loop through, search for strings

    applications = [
        "redis",
        "rabbitmq",
    ]
    cmd = "/usr/bin/docker container list"
    try:
        # will fail if docker is not already installed and running
        output = subprocess.check_output(
            cmd, shell=True, stderr=subprocess.STDOUT
        ).decode("UTF-8")
    except subprocess.CalledProcessError as err:
        hookenv.log("Listing containers failed with {}".format(err))
        return []

    monitor_applications = [app for app in applications if app in output]
    return monitor_applications


@hook("update-status")
def update_status():
    """Detect whether there are any docker containers requiring configuration."""
    monitor_applications = detect_docker_containers()
    for app in monitor_applications:
        flag_name = "nrpe_{}".format(app)
        if not is_flag_set(flag_name + ".configured"):
            set_flag(flag_name + ".required")


@when("contrail-nrpe.configured")
@when("nrpe_redis.required")
@when_not("nrpe_redis.configured")
def setup_redis_nrpe():
    """Install and configure the Redis NRPE plugin."""
    nrpe_setup = nrpe.NRPE(primary=False)
    nrpe_check = redis.RedisNrpe()
    nrpe_check.install()
    nrpe_check.configure_nrpe_check(nrpe_setup)
    set_flag("nrpe_redis.configured")


@when("contrail-nrpe.configured")
@when("nrpe_rabbitmq.required")
@when_not("nrpe_rabbitmq.configured")
def setup_rabbitmq_nrpe():
    """Install and configure the Contrail RabbitMQ NRPE plugin."""
    nrpe_setup = nrpe.NRPE(primary=False)
    config = hookenv.config()
    cmd = ""
    # If value of queue_thresholds is incorrect we want the hook to fail
    for item in yaml.safe_load(config["queue_thresholds"]):
        cmd += ' -c "{}" "{}" {} {}'.format(*item)
    cmd += " /var/lib/misc/rabbitmq-check/rabbitmq_queue_stats.dat"
    nrpe_check = contrailrabbit.ContrailRabbitMqCheck(nrpe_opts=cmd)
    nrpe_check.install()
    nrpe_check.configure_nrpe_check(nrpe_setup)
    set_flag("nrpe_rabbitmq.configured")


@when("config.changed")
def config_changed():
    """Handle configuration changes."""
    clear_flag("contrailnrpe.configured")
    monitor_applications = detect_docker_containers()
    for app in monitor_applications:
        clear_flag("nrpe_{}.configured".format(app))


@when("nrpe-external-master.available")
@when_not("contrail-nrpe.configured")
def configure_nrpe():
    """Configure NRPE checks."""
    # The above docstring is based upon what this function seems to be
    # intended to do, but I don't presently see it doing much other
    # than waiting until /var/lib/nagios exists.

    if not os.path.exists("/var/lib/nagios"):
        status.waiting("Waiting for nrpe package installation")
        return

    status.maintenance("Configuring nrpe checks")

    status.active("ready")
    set_flag("contrail-nrpe.configured")


@when_not("nrpe-external-master.available")
@when("contrailnrpe.configured")
def remove_nrpe_checks():
    """Remove NRPE checks."""
    status.maintenance("Removing nrpe checks")
    clear_flag("contrailnrpe.configured")
    # TODO
