"""Tests for the Redis plugin."""

import os
import sys
import tempfile
import unittest
import unittest.mock as mock
from pathlib import Path

sys.path.append("files/plugins")
from check_redis import check_redis_status  # noqa: E402

# some dep layers only exists in the built charm; mock them out before
# the imports since those depend on post-build charms.* layers
ch_nrpe_mock = mock.Mock()
sys.modules["charmhelpers.contrib.charmsupport"] = ch_nrpe_mock
hookenv_mock = mock.Mock()
sys.modules["charmhelpers.core.hookenv"] = hookenv_mock

from contrailnrpe import redis  # noqa: E402

TESTDIR = Path(__file__).parent
TESTDATA = TESTDIR / "testdata"
CHARMDIR = TESTDIR.parent.parent


class TestCheckRedis(unittest.TestCase):
    """Tests of check_redis_status()."""

    @mock.patch("subprocess.check_output")
    def test_parse_output_ok(self, mock_subprocess):
        """Happy-path test for check_redis_status()."""
        mock_result = b"""
                      # Server
                      run_id:2e09272841bab6dc9a201e9e714f1aee37dda4be

                      tcp_port:6379
                      uptime_in_seconds:64489
                      uptime_in_days:0
                      """
        mock_subprocess.return_value = mock_result
        expected = "OK: Contrail status OK"
        status, message = check_redis_status()
        self.assertTrue(status == 0 and expected in message)

    @mock.patch("subprocess.check_output")
    def test_check_uptime(self, mock_subprocess):
        """Test check_redis_status() uptime critical error."""
        mock_result = b"""
                      # Server
                      run_id:2e09272841bab6dc9a201e9e714f1aee37dda4be

                      tcp_port:6379
                      uptime_in_seconds:2
                      uptime_in_days:0
                      """
        mock_subprocess.return_value = mock_result
        expected = "CRITICAL: Uptime for Redis is less than 300"
        status, message = check_redis_status()
        self.assertTrue(status == 2 and expected in message)


class TestRedis(unittest.TestCase):
    """Tests of redis.RedisNrpe."""

    @classmethod
    def setUpClass(cls) -> None:
        """Class setup method."""
        hookenv_mock.charm_dir.return_value = CHARMDIR
        cls.tmp = tmp = Path(tempfile.mkdtemp())
        cls.rootfs = tmp / "rootfs"
        cls.helper = redis.RedisNrpe()
        cls.helper.NRPE_PLUGINS_DIR = str(cls.rootfs / "usr/local/lib/nagios/plugins")
        cls.helper.NRPE_PLUGINS_UID = euid = os.geteuid()  # noqa: F841
        cls.helper.NRPE_PLUGINS_GID = egid = os.getegid()  # noqa: F841
        cls.helper.SUDOERS_DIR = str(cls.rootfs / "etc/suders.d")
        cls.helper.SUDOERS_UID = euid = os.geteuid()  # noqa: F841
        cls.helper.SUDOERS_GID = egid = os.getegid()  # noqa: F841
        cls.helper.SUDOERS_MODE = 0o100660  # so we can delete it

    def test_install(self):
        """Test plugin installation."""
        self.helper.install()
        nrpe_cmd = self.rootfs / "usr/local/lib/nagios/plugins/check_redis"
        with nrpe_cmd.open() as f:
            self.assertTrue("def check_redis_status" in f.read())

    def test_remove(self):
        """Test plugin removal."""
        self.helper.install()
        self.helper.remove()
        nrpe_cmd = self.rootfs / "usr/local/lib/nagios/plugins/check_redis"
        self.assertFalse(nrpe_cmd.exists())
